@@ -62,9 +62,6 @@ initially by the command line -sedit switch."] \
 "If enabled, after editing with external editor, exmh will bring the message
 up in sedit.  This is so you can initially create the message with your
 favorite editor, and then pop into to sedit for richtext or attachments."] \
-    [list editor(spell) spellCmd "exmh-async xterm -e ispell" {Spell command} \
-"The spell command specifies a program used to spell-check mail messages. 
-The filename will be appended to the command."] \
         [list editor(mhn) mhnCmd "mhn" {MHN command} \
 "The mhn command specifies a program used to reformat
 a message with the MH mhn program.
@@ -311,7 +308,7 @@ proc EditDialogDone {act id {hide hide}} {
     if [string match hide $hide] {
 	Exwin_Dismiss .edit$id nosize
     }
-    Edit_Done $act $id
+    Edit_Done $act $id {}
 }
 
 proc EditStart { draft {type prog} } {
@@ -352,7 +349,7 @@ proc EditStart { draft {type prog} } {
 		    }
 		}
 	    }
-	    Exmh_Debug $id mxReset
+	    Exmh_Debug mxReset $id
 	    if [catch {send $id mxReset}] {
 		if [catch {send $exmh(editInterp) {set mxVersion}}] {
 		    Exmh_Status "Starting mxedit..." warn
@@ -421,10 +418,10 @@ proc Edit_Ident { interpName } {
 }
 
 # The following is invoked by remote editor interpreters
-proc EditDone {act msg} {
-    Edit_Done $act $msg
+proc EditDone {act msg argu} {
+    Edit_Done $act $msg $argu
 }
-proc Edit_Done {act {msg cur}} {
+proc Edit_Done {act {msg cur} argu} {
     # Commit or abort an edit session
     global mhProfile exmh env editor pgp
     if {$msg == "cur"} {
@@ -434,7 +431,7 @@ proc Edit_Done {act {msg cur}} {
 	    return
 	}
     }
-    Exmh_Debug action = $act msg = $msg
+    Exmh_Debug Edit_Done action = $act msg = $msg
     if ![regexp {^[0-9]+$} $msg] {
 	# Message not in the drafts folder
 	set path $mhProfile(path)/$msg
@@ -451,7 +448,7 @@ proc Edit_Done {act {msg cur}} {
 	    }
 	    set anno [Mh_AnnoEnviron $msg]
 	    Exmh_Debug Edit_Done send: anno=$anno
-	    Exmh_Debug "Mh_Send [time {set code [catch {Mh_Send $msg} err2]}]"
+	    Exmh_Debug "Mh_Send [time {set code [catch {Mh_Send $msg $argu} err2]}]"
 	    if $code {
 		# move the orig message back to the draft if it exists
 		Mhn_RenameOrig $msg
