@@ -187,7 +187,7 @@ Any given mail address can match a range of face images, from most specific (suc
         {faces(xFaceEnabled) xFaceEnabled ON {Use X-Face headers}
 	"If a message contains an X-Face header, display the image encoded into the header."}
 
-	{faces(xfaceProg) xfaceProg {} {X-Face command}
+	{faces(xfaceProg) xfaceProg {uncompface -X} {X-Face command}
 "A command to convert an X-Face: header line into a displayable X11 bitmap.
 
 If you've patch uncompface, this can be:	uncompface -X
@@ -430,6 +430,10 @@ The text/enriched formatting is done for both OnSend and OnType
 formatting choices.
 This Preference setting chooses the default behaviour, which
 you can change with the Format menu entry."}
+    {sedit(xMailHeader) seditxMailHeader ON {Include X-Mailer Header}
+"Mail sent can include a header indicating the version of the mail
+program used to create t he mail.  You can allow exmh to include this
+header, or retain anonymity using this preference item."}
     {sedit(mhnDefault) seditAttemptMhn OFF {Attempt mhn default}
 "Sedit can send your message thru mhn in order to expand its #
 MIME formating directives (see the man page about mhn for details).
@@ -475,20 +479,42 @@ this is the character set used for text content types"}
 "If the type of a file cannot be determined from the mime.types file
 and from the file -m program, use this as the default MIME type
 when attaching files."}
-    {sedit(spell) seditSpell {CHOICE spell ispell custom} {spell program}
-"This chooses the spell program used by the built-in editor.
-Use custom if you want to define the program explicitly."}
-    {editor(spell) spellCmd {exmh-async xterm -e ispell} {custom spell command}
-"NOTE: this affects the same internal variable as the
-Spell Command in Editor preferences.
+    {sedit(spell) seditSpell {CHOICE spell ispell custom interactive} {Spell program}
+"This chooses the spell program used by the built-in editor. \"spell\" selects the standard
+spell program (which is not interactive). \"ispell\" enables checking 
+with the ispell program. \"custom\" runs the program set in the next preference.
+\"interactive\" enables the I-Spell module, which has its own preference page."}
+    {editor(spell) spellCmd {exmh-async xterm -e ispell} {Custom spell command}
+"This option is ignored unless you chose \"custom\" as the spell program option.
 
 There are two flavors of spell programs.  If your spell program just
 prints out the misspelled words, then just specify it directly.
-Examples include the standard \"spell\" program.  If you spell program
-is interactive, then run it from exmh-async:
-exmh-async xterm -e ispell
-(This is faked - exmh-async isn't really used.  Instead a temporary
-wish script is used.)"}
+Examples include the standard \"spell\" program.  If your spell program
+is interactive, then prefix it with exmh-async:
+ exmh-async xterm -e ispell
+Your spell program will be given the file name to check as argument."}
+    {sedit(notifySuccess) seditNotifySuccess OFF {Notify on sucessful delivery}
+"This lets you say whether you would like to be informed (via an email
+message) when your mail has been delivered.  For it to work you will
+need a hacked over version of MH, and an SMTP server which supports
+the IETF's Delivery Status Notification SMTP extension!"}
+    {sedit(notifyFailure) seditNotifyFailure OFF {Notify if delivery fails}
+"This lets you say whether you would like to be informed (via an email
+message) when your mail has been not been delivered because of an
+error.  For it to work you will need a hacked over version of MH, and
+an SMTP server which supports the IETF's Delivery Status Notification
+SMTP extension!"}
+    {sedit(notifyDelay) seditNotifyDelay OFF {Notify if delivery is delayed}
+"This lets you say whether you would like to be informed (via an email
+message) when delivery of your mail has been been delayed.  For it to
+work you will need a hacked over version of MH, and an SMTP server
+which supports the IETF's Delivery Status Notification SMTP extension!"}
+    {sedit(notifyRet) seditNotifyRet OFF {Return full message}
+"This lets you say whether you would like delivery status notifications
+to take the form of full copies of the original message, or just the
+headers - the default.  For it to work, you will need a hacked over
+version of MH, and an SMTP server which supports the IETF's Delivery
+Status Notification SMTP extension!"}
     {sedit(autosaveInterval)   seditAutosaveInterval 60 {Seconds between auto saves}
 "This is the interval, in seconds, between automatic saves of 
 the message being edited.  Set to 0 to disable autosaving."}
@@ -625,8 +651,11 @@ Here you can select, which version to use in these cases.
 
     Preferences_Add "General PGP Interface" "" {
     {pgp(keeppass) pgpKeepPass ON {Keep PGP passphrase}
-"Exmh tries to remember your passphrase between PGP
-invocations. But the passphrase is then kept in a global
+"If this option is active, Exmh tries to remember your passphrase between PGP
+invocations. This option is disabled if you use GnuPG and
+select the gpgUseAgent option.
+If pgpExtPass is on, exmh will use an external passphrase
+cache program; otherwise the passphrase is then kept in a global
 variable, which is not safe, because of \"send\"'s power.
 If you turn this feature off, exmh will use xterm to run
 pgp so that it doesn't have to deal with the passphrase at all." }
@@ -645,7 +674,7 @@ this time period, in minutes, has elapsed." }
 {pgp(extpass) pgpExtPass OFF {Use external passphrase cache}
 "If this is enabled, then exmh will use an external program to retrieve 
 your passphrase when needed. pgpKeepPass and pgpPassTimeout will 
-be ignored."}
+be ignored. pgpExtPass has no effect if pgpKeepPass is not active. "}
 {pgp(getextcmd) pgpGetExtCmd {/usr/bin/q-client get %s} {Method to query external passphrase cache}
 "This external program is used to retrieve the passphrase for your key,
 if pgpExtPass is active. The passphrase is expected on stdout.
@@ -695,6 +724,7 @@ The key id is substituted with %s (using format)." }
 #               (max hits (per folder))
 #  -L {limit}:0:{flimit} : limit per file	: since 3.0:glimpse(maxHitsMsg)
 #               (max hits per message)
+#  -W : AND scope whole file			: since 2.0:glimpse(andScope) 
 
 proc Glimpse_Init {} {
 	global glimpse
@@ -746,7 +776,18 @@ If you have a single large glimpse index, it applies
 to the whole search.
 
 The maximum # of matches can be changed on the fly in the 'Glimpse Window'
-in the menu 'opts'"} } }
+in the menu 'opts'"} } 
+
+Preferences_Add "Glimpse" "" {
+    {glimpse(andScope) glimpseAndScope ON {AND scope is whole file}
+	"If set, the scope for boolean AND is the whole file.
+The default scope of Boolean AND queries is a single line, and 
+the query 'good;bad' will return all lines containing both 'good' 
+and 'bad'. This option changes the scope of Booleans to be the 
+whole file. With the option enabled, the query 'good;bad' would 
+return all lines containing 'good' or 'bad' if the file contains 
+matches for both terms."}}
+}
 
 if {$glimpse(version) >= 3.0} {
     Preferences_Add "Glimpse" "" {
