@@ -101,9 +101,11 @@ proc Ftoc_Update { numMsgs } {
 proc Ftoc_Bindings { w } {
     # Bindings for the ftoc text widget
 
+    bind $w <Configure> FtocDeduceSize
+
     # The TScroll binding to too general.
     # We'll do our own scroll bindings here.
-    bindtags $w [list $w]
+    bindtags $w [list $w Command]
 
     # Button-1 starts selection range
     bind $w <Button-1> {
@@ -138,6 +140,18 @@ proc Ftoc_Bindings { w } {
     bind $w <B2-Motion> {WidgetTextDragto %W %y $exwin(scrollSpeed)}
 
     Drag_Attach $w FtocDragSelect Shift 3
+
+}
+proc FtocDeduceSize {} {
+    global exwin
+    if {$exwin(toplevelMsg)} {
+	set top [lindex [split [$exwin(ftext) index @0,0] .] 0]
+	set h [winfo height $exwin(ftext)]
+	set w [winfo width $exwin(ftext)]
+	set ix [$exwin(ftext) index @[expr $h-1],[expr $w-1]]
+	set bottom [lindex [split $ix .] 0]
+	set exwin(ftextLines) [expr $bottom-$top]
+    }
 }
 proc FtocRangeStart { lineno } {
     # For normal button-down "start a selection"
@@ -387,7 +401,7 @@ proc Ftoc_LinesHighlight { linenos } {
 	return
     }
     WidgetTextYview $exwin(ftext) -pickplace [lindex $linenos 0].0
-    update idletasks
+#    update idletasks
     foreach lineno $linenos {
 	set newtag range
 	foreach tag [$win tag names $lineno.0] {
@@ -465,8 +479,8 @@ proc Ftoc_FindMsg { msgid } {
     if {$msgid == {}} {
         return {}
     }
-    if {![catch {set msgtolinecache($msgid)} lineno]} {
-        return $lineno
+    if {[info exist msgtolinecache($msgid)]} {
+        return $msgtolinecache($msgid)
     }
     if !$ftoc(displayValid) {
         #
@@ -496,16 +510,27 @@ proc Ftoc_FindMsg { msgid } {
     while (1) {
         if {$msgid > $maxmsgid || $msgid < $minmsgid} {
             Exmh_Status "Cannot find $msgid ($minmsgid,$maxmsgid)" warn
-            set msgtolinecache($msgid) {}
+            if {[info exist msgtolinecache($msgid)]} {
+                unset msgtolinecache($msgid)
+            }
             return {} ;# new message not listed
         }
         if {$maxlineno == $minlineno} {
-            set msgtolinecache($msgid) {}
+            if {[info exist msgtolinecache($msgid)]} {
+                unset msgtolinecache($msgid)
+            }
             return {}   ;# not found
         }
+        # Original binary search
         #set nextlineno [expr int(($maxlineno+$minlineno)/2)]
         # Don't divide in two, guestimate where the line might be instead
         set nextlineno [expr int($minlineno+1+($msgid-$minmsgid)*($maxlineno-$minlineno-2)/($maxmsgid-$minmsgid))]
+        if {$nextlineno < $minlineno} {
+          set nextlineno $minlineno
+        }
+        if {$nextlineno > $maxlineno} {
+          set nextlineno $maxlineno
+        }
         set nextmsgid [Ftoc_MsgNumber $nextlineno]
         # Note that a side effect of Ftoc_MsgNumber was to put this entry in 
 	# the cache,so we don't have to do it here.
@@ -516,9 +541,13 @@ proc Ftoc_FindMsg { msgid } {
             set maxmsgid $nextmsgid
         } elseif {$minlineno == $nextlineno} {
             Exmh_Status "Cannot find $msgid" warn
-            set msgtolinecache($msgid) {}
+            if {[info exist msgtolinecache($msgid)]} {
+                unset msgtolinecache($msgid)
+            }
             return {} ;# new message not listed
-        } else {
+        } elseif {$nextmsgid == ""} {
+            error "Failed to find a message number on line $nextlineno, end is [$::exwin(ftext) index end]\n[$::exwin(ftext) get $nextlineno.0 $nextlineno.end]"
+        } else { 
             set minlineno $nextlineno
             set minmsgid $nextmsgid
         }
@@ -534,7 +563,7 @@ proc Ftoc_ClearMsgCache {} {
     }
 }
 proc Ftoc_MsgNumbers { linenos } {
-    global exwin linetomsgcache msgtolinecache
+    global exwin
     set msgids {}
     foreach lineno $linenos {
 	set msgid [Ftoc_MsgNumber $lineno]
@@ -546,12 +575,14 @@ proc Ftoc_MsgNumbers { linenos } {
 }
 proc Ftoc_MsgNumber { lineno } {
     global exwin linetomsgcache msgtolinecache
-    if [catch {set linetomsgcache($lineno)} msgid] {
-        if [catch {$exwin(ftext) get $lineno.0 $lineno.end} line] {
-            set linetomsgcache($lineno) {}
-            return {}
-        }
-        set msgid [Ftoc_MsgNumberRaw $line]
+    if {[info exist linetomsgcache($lineno]} {
+        return $linetomsgcache($lineno)
+    }
+    if [catch {$exwin(ftext) get $lineno.0 $lineno.end} line] {
+        return {}
+    }
+    set msgid [Ftoc_MsgNumberRaw $line]
+    if {$msgid != {}} {
         set msgtolinecache($msgid) $lineno
         set linetomsgcache($lineno) $msgid
     }
@@ -674,22 +705,35 @@ proc Ftoc_InitSequences { w } {
 	$w tag raise $seq
     }
 }
+
+# Highlight a set of messages (or all in the folder) that belong
+# to a sequence.  If msgsids is null, then we only work on that
+# subset of the folder.  Otherwise we highlight all the messages
+# in the folder that are in the sequence.
+
 proc Ftoc_ShowSequence { seq {msgids {}} } {
-    global exwin exmh
+    global exwin exmh mhProfile
+Exmh_Debug Ftoc_ShowSequence $seq msgids $msgids
     set seqids [Seq_Msgs $exmh(folder) $seq]
     if {$msgids != {}} {
 	foreach msg $msgids {
 	    set lineno [Ftoc_FindMsg $msg]
-	    if {[lsearch -exact $seqids $msg] == -1} {
-		$exwin(ftext) tag remove $seq $lineno.0 $lineno.end
-	    } else {
-		$exwin(ftext) tag add $seq $lineno.0 $lineno.end
+	    if {$lineno != {}} {
+		if {[lsearch -exact $seqids $msg] == -1} {
+		    $exwin(ftext) tag remove $seq $lineno.0 $lineno.end
+		} else {
+		    $exwin(ftext) tag add $seq $lineno.0 $lineno.end
+		}
 	    }
 	}
     } else {
 	$exwin(ftext) tag remove $seq 1.0 end
-	foreach lineno [Ftoc_FindMsgs $seqids] {
-	    $exwin(ftext) tag add $seq $lineno.0 $lineno.end
+        if {$seq == $mhProfile(unseen-sequence)} {
+            FtocShowUnseen $seqids
+        } else {
+            foreach lineno [Ftoc_FindMsgs $seqids] {
+                $exwin(ftext) tag add $seq $lineno.0 $lineno.end
+            }
 	}
     }
 }
@@ -697,6 +741,7 @@ proc Ftoc_ShowSequence { seq {msgids {}} } {
 proc Ftoc_ShowSequences { {msgids {}} } {
     global exwin exmh
     if {$msgids == {}} {
+Exmh_Debug Ftoc_ShowSequences msgids null
 	set seqs [option get . sequences {}]
 	set hiddenseqs [option get . hiddensequences {}]
 	foreach seq $seqs {
@@ -704,11 +749,53 @@ proc Ftoc_ShowSequences { {msgids {}} } {
 		$exwin(ftext) tag remove $seq 1.0 end
 	    }
 	}
+    } else {
+Exmh_Debug Ftoc_ShowSequences msgids $msgids
     }
     foreach seq [Mh_Sequences $exmh(folder)] {
-	Ftoc_ShowSequence $seq $msgids
+        Ftoc_ShowSequence $seq $msgids
     }
 }
+
+# This is optimized for the unseen sequence, which tends to
+# cluster at the end of a folder, and get big
+
+proc FtocShowUnseen { unseen } {
+    global exwin flist
+    if {[llength $unseen] > 0} {
+Exmh_Debug FtocShowUnseen $unseen
+	set end [$exwin(ftext) index end]
+	set line [lindex [split $end .] 0]
+	set msgNum 0
+	for {} {$line > 0} {incr line -1} {
+	    set msgNum [Ftoc_MsgNumber $line]
+	    set i [lsearch $unseen $msgNum]
+	    if {$i >= 0} {
+		$exwin(ftext) tag add unseen $line.0 $line.end
+		set unseen [lreplace $unseen $i $i]
+		if {[llength $unseen] == 0} {
+		    return 1
+		}
+	    }
+	}
+        # Here is some code from the old Ftoc_ShowUnseen that
+        # I don't think we need any more
+        if {0} {
+          # Repair bogus unseen sequences
+          # msgNum is the smallest message number, but it might not be
+          # the first message in the folder because of short scans
+          # Anything in the unseen sequence above msgNum is probably wrong
+          # and can result from races with the background process
+          foreach id $unseen {
+	    if {$id > $msgNum} {
+                # This API doen't exist anymore
+		Flist_MsgSeenXXX $id
+	    }
+          }
+	}
+    }
+}
+
 proc Ftoc_RescanLine { ix {plus none} } {
     global exmh exwin ftoc
     if [catch {
@@ -1092,6 +1179,7 @@ proc FtocCommit {tagname commitProc {copyCommitProc {}} } {
     set delmsgs {}
     set curid [file tail $msg(path)]
     set pairs [FtocMakeReversePairs [$exwin(ftext) tag ranges $tagname]]
+    set seqdelmsgs {}
     foreach range $pairs {
 	set c0 [lindex $range 0]
 	set ce [lindex $range 1]
@@ -1100,12 +1188,12 @@ proc FtocCommit {tagname commitProc {copyCommitProc {}} } {
 	set F {}
 	set delline 0	;# Nuke display line
 	foreach tag [$exwin(ftext) tag names $c0] {
+            # Build up a list of moved or copied messages
+            # Note that the original order of the messages is maintained,
+            # (We are going from bottom to top thru the display.)
+            # The scan lines are reversed, which is handled by Scan_Move.
 	    if {([llength $tag] == 2) && ([lindex $tag 0] == "moved")} {
 		set F [lindex $tag 1]
-		# Build up a list of moved messages
-		# Note that the original order of the messages is maintained,
-		# (We are going from bottom to top thru the display.)
-		# The scan lines are reversed, which is handled by Scan_Move.
 		if ![info exists movemsgs($F)] {
 		    set movemsgs($F) $msgid
 		} else {
@@ -1116,10 +1204,6 @@ proc FtocCommit {tagname commitProc {copyCommitProc {}} } {
 	    }
 	    if {([llength $tag] == 2) && ([lindex $tag 0] == "copied")} {
 		set F [lindex $tag 1]
-		# Build up a list of moved messages
-		# Note that the original order of the messages is maintained,
-		# (We are going from bottom to top thru the display.)
-		# The scan lines are reversed, which is handled by Scan_Move.
 		if ![info exists copymsgs($F)] {
 		    set copymsgs($F) $msgid
 		} else {
@@ -1133,7 +1217,9 @@ proc FtocCommit {tagname commitProc {copyCommitProc {}} } {
 	    lappend delmsgs $msgid
 	    set delline 1
 	}
-	Seq_Del $exmh(folder) $mhProfile(unseen-sequence) $msgid	;# in case deleted or moved w/out viewing
+	lappend seqdelmsgs $msgid					;# in case deleted or moved w/out viewing
+	#Exmh_Debug FtocCommit added $msgid to be deleted from unseen-sequence
+
 	if {$delline} {
 	    $exwin(ftext) delete $c0 "$ce + 1 chars"
 	    set ftoc(displayDirty) 1
@@ -1158,6 +1244,12 @@ proc FtocCommit {tagname commitProc {copyCommitProc {}} } {
 	}
 	incr ftoc(changed) -1
     }
+
+    if {$seqdelmsgs != {}} {
+	Exmh_Debug FtocCommit deleting msgs from $mhProfile(unseen-sequence) $seqdelmsgs
+	Seq_Del $exmh(folder) $mhProfile(unseen-sequence) $seqdelmsgs
+    }
+
     if {$delmsgs != {}} {
 	Exmh_Status "$commitProc $delmsgs"
 	if [catch {
@@ -1218,6 +1310,9 @@ proc FtocMakeReversePairs { list } {
 proc Ftoc_MoveFeedback { msgid } {
     global exwin ftoc
     set lineno [Ftoc_FindMsg $msgid]
+    if {$lineno == ""} {
+        return
+    }
     set msg [Exmh_OldStatus]
     foreach tag [$exwin(ftext) tag names $lineno.0] {
 	if [regexp {moved (.+)} $tag match folder] {
@@ -1402,3 +1497,8 @@ proc FtocToggleSequence { seq } {
 	Ftoc_ShowSequence $seq $selmsgids
     }
 }
+
+# exmh-2.5 APIs
+# Ftoc_ColorConfigure
+# Ftoc_MarkSeen
+# Ftoc_ShowUnseen
