@@ -1,4 +1,4 @@
-# $Header: /cvsroot/exmh/exmh/lib/glimpse.tcl,v 1.4 2000/11/17 22:55:10 welch Exp $
+# $Header: /cvsroot/exmh/exmh/lib/glimpse.tcl,v 1.7 2008/06/18 10:01:27 az143 Exp $
 #
 # link glimpse to exmh to provide full text searching within and
 # across folders faster but less thoroughly than pick
@@ -63,7 +63,11 @@ proc Glimpse_Startup {} {
     }
     # set default variable values
     set glimpse(search) ""
+    if $glimpse(andScope) {
+	set glimpse(andscope) "-W"
+    } else {
     set glimpse(andscope) ""
+    }
     set glimpse(giall) 0
 
     if {$glimpse(maxErrors) == "none"} {
@@ -300,7 +304,7 @@ proc Glimpse_Start {} {
 	if [catch {
 	    open "| $glimpse(path)/glimpse $opts -H $idir -e \"$glimpse(search)\"" r
 	} x] {
-	    Exmh_Debug $x
+	    Exmh_Debug Glimpse error $x
 	    set result $x
 	} else {
 	    set glimpse(result) {}
@@ -485,7 +489,7 @@ proc Glimpse_Index { } {
     }
 
     if ![file isdirectory $mhProfile(path)/.glimpse] {
-	if [catch {exec mkdir $mhProfile(path)/.glimpse} err] {
+	if [catch {file mkdir $mhProfile(path)/.glimpse} err] {
 	    set glimpse(info) $err
 	    return
 	}
@@ -638,7 +642,7 @@ proc MakeDir { path } {
 	if ![file isdirectory [file dirname $path]] {
 	    MakeDir [file dirname $path]
 	}
-	exec mkdir $path
+	file mkdir $path
     }
 }
 
@@ -662,7 +666,7 @@ proc Glimpse_Batch { i cmd } {
 	set myname [winfo name .]
 	puts $out "catch \{exec $cmd\} out"
 	puts $out "catch \{send [list $myname] \[list Glimpse_BatchDone \$out\]\}"
-	puts $out "exec rm $path"
+	puts $out "file delete -force $path"
 	puts $out exit
 	close $out
 	set glimpse(done) {}
@@ -718,7 +722,7 @@ proc Glimpse_Delete { f } {
 	    append rmsg "$info"
 	} else {
 	    append rmsg "glimpse index files deleted"
-	    catch {exec rmdir $idir}
+	    catch {file delete $idir}
 	}
     }
     return $rmsg
