@@ -157,10 +157,14 @@ proc HMinit_win {win} {
 	bind $win <Configure> {
 	    %W tag configure last -spacing3 %h
 	}
-	bind TScroll <Key-Next> {%W yview scroll 1 page}
-	bind TScroll <Key-Prior> {%W yview scroll -1 page}
-	bind TScroll <Key-Home> {%W see 1.0}
-	bind TScroll <Key-End> {%W see end}
+	if {0} {
+	    # These are already on the exmh window
+
+	    bind TScroll <Key-Next> {%W yview scroll 1 page}
+	    bind TScroll <Key-Prior> {%W yview scroll -1 page}
+	    bind TScroll <Key-Home> {%W see 1.0}
+	    bind TScroll <Key-End> {%W see end}
+	}
 
 }
 
@@ -377,6 +381,11 @@ proc !HMtag_title {win param text} {
 	set data ""
 }
 
+# remove <style> block (since we don't process it, yet)
+proc HMtag_style {win param text} {
+	upvar $text data
+	set data ""
+}
 
 proc HMtag_br {win param text} {
 	upvar #0 HM$win var
