@@ -27,9 +27,11 @@ proc HMlink_setup {win htag} {
     regsub -all % $href %% href2
     $win tag configure $tag -foreground [Widget_ColorDefault $win c_link] -underline 1
     $win tag bind $tag <Enter> \
-	    [list HMlink_feedback $win hand2 $href2 $tag c_alink]
+	    [list HMlink_feedback $win hand2 "X-URL:\t$href2" $tag c_alink]
     $win tag bind $tag <Leave> \
 	    [list HMlink_feedback $win [option get $win cursor Text] "" $tag c_link]
+    # make the link clickable
+    $win tag bind $tag <ButtonRelease-1> [list URI_StartViewer $href2 ]
 
     # Add to history so it shows up in URL chooser dialog
     global History
@@ -79,13 +81,13 @@ proc Url_Hit? {win x y} {
 }
 proc Url_Hit {win x y} {
     dputs Url_Hit $x $y
-    lassign {href name} [UrlGetLink $win $x $y]
+    exmh_lassign {href name} [UrlGetLink $win $x $y]
     Html_HistoryAdd $win $href
     Frame_Display $win $name $href
 }
 # This is like Url_Hit for regular hits, but it opens a new window.
 proc Url_HitNew {win x y} {
-    lassign {href target} [UrlGetLink $win $x $y]
+    exmh_lassign {href target} [UrlGetLink $win $x $y]
     Url_DisplayNew $href $win
 }
 
@@ -224,14 +226,14 @@ proc HMlink_callback {win href {query ""}} {
 	    }
 	    mailto {
 		if {[string match exmh* [tk appname]]} {
-		    Sedit_Mailto $href
+		    Msg_Mailto $href
 		} else {
 		    set interps [winfo interps]
 		    set ix [lsearch -regexp $interps {exmh( #.)?$}]
 		    if {$ix >= 0} {
 			set exmh [lindex $interps $ix]
 			Status $win "Using $exmh to send mail"
-			send $exmh [list Sedit_Mailto $href]
+			send $exmh [list Msg_Mailto $href]
 		    } else {
 			regsub mailto: $href {} address
 			Status $win "Please send mail to $address"
