@@ -134,7 +134,7 @@ proc Inc_PresortMultidrop {} {
     # that goes into MyIncTmp will get sorted later.
     global mhProfile
     if ![file isdirectory $mhProfile(path)/MyIncTmp] {
-	exec mkdir $mhProfile(path)/MyIncTmp
+	file mkdir $mhProfile(path)/MyIncTmp
     }
     # Inc, but don't fiddle with scan listings
     Inc_All 0
@@ -167,7 +167,7 @@ proc Inc_Inbox {} {
     BgRPC Inc_InboxFinish $inbox $incout Flist_Done
 }
 proc Inc_Expect {cmd} {
-    global inc exmh pop
+    global inc exmh pop install
     if {![info exist pop(password)]} {
 	# No password implies no POP host
 	return $cmd
@@ -176,7 +176,7 @@ proc Inc_Expect {cmd} {
     # Drop the leading "exec", and splice in the -host argument
 
     set cmd [lrange $cmd 1 end]
-    set cmd [concat [list exec inc.expect] $cmd]
+    set cmd [concat [list exec $install(dir,bin)/inc.expect] $cmd]
     if {[lsearch -exact $cmd -host] < 0} {
 	lappend cmd -host $inc(pophost)
     }
@@ -223,7 +223,7 @@ proc Inc_Presort {{doinc 1}} {
     global exmh mhProfile env inc pop
     # Use inc to copy stuff into a temp directory
     if ![file isdirectory $mhProfile(path)/MyIncTmp] {
-	exec mkdir $mhProfile(path)/MyIncTmp
+	file mkdir $mhProfile(path)/MyIncTmp
     }
     if {$doinc} {
 	set cmd [list exec inc +MyIncTmp -silent]
@@ -278,7 +278,7 @@ proc Inc_Presort {{doinc 1}} {
 		# (can't stat .) lead to this behavior
 		Exmh_Status "$file - $err - check MyIncErrors"
 		if ![file isdirectory $mhProfile(path)/MyIncErrors] {
-		    exec mkdir $mhProfile(path)/MyIncErrors
+		    file mkdir $mhProfile(path)/MyIncErrors
 		}
 		Mh_Refile MyIncTmp [file tail $file] MyIncErrors 
 	    } else {
@@ -303,7 +303,7 @@ proc Inc_Presort {{doinc 1}} {
 	}
     }
     File_Delete $mhProfile(path)/MyIncTmp/$mhProfile(mh-sequences)
-    catch {exec rmdir $mhProfile(path)/MyIncTmp}
+    catch {file delete $mhProfile(path)/MyIncTmp}
 
     Flist_FindSeqs		;# Needed to set new message state.
     # This after breaks a potential deadlock:
@@ -317,7 +317,7 @@ proc Inc_Presort {{doinc 1}} {
 proc Inc_PresortFinish {} {
     global exmh ftoc mhProfile
     LOG Inc_PresortFinish
-    Mh_Folder $exmh(folder)	;# presort inc has changed this to MyIncTmp
+    Mh_FolderFast $exmh(folder)	;# presort inc has changed this to MyIncTmp
     if {$ftoc(displayValid) && [Seq_Count $exmh(folder) $mhProfile(unseen-sequence)] > 0} {
 	Label_Folder $exmh(folder)
 	Scan_Folder $exmh(folder) $ftoc(showNew)
