@@ -25,6 +25,14 @@ proc Mh_Init {} {
 	# 'repl -- version [compiled etc etc]' - catch version
 	catch {MhExec repl -version} d
 	regexp {.*-- *([^ ]*)[ ]} $d {} exmh(mh_vers) 
+	# See if it's an nmh patched to support setting info for rfc3461 DSNs
+	set exmh(nmh_dsn) 0
+	catch {MhExec send -help} d
+	set d1 [ split $d "\n"]
+	foreach line $d1 {
+	    regexp {envid} $line d2
+	    if [info exists d2] { set exmh(nmh_dsn) 1 }
+	}
     } else {
 	# UCI MH - 'version: .*'
 	# weirdness - 6.8 puts 'version (build on ...)', 6.6 (blech) doesnt.
@@ -104,6 +112,7 @@ proc MhBackup {} {
 proc MhExec { args } {
     global mhProfile
     Audit $args
+    set args [join $args]
     if {[catch {eval exec $args} result]} {
 	global errorInfo
 	if {[regexp {exmhcontext is poorly formatted} $result]} {
@@ -332,7 +341,10 @@ proc Mh_Folder { f } {
 	}
     }
 }
-proc Mh_FolderNew { f } {
+proc Mh_FolderNew { f } {       ;# Not sure if this name is still used
+    Mh_SetContext Current-Folder $f
+}
+proc Mh_FolderFast { f } {
     Mh_SetContext Current-Folder $f
 }
 proc Mh_SetContext { key value } {
@@ -350,7 +362,7 @@ proc Mh_SetContext { key value } {
 	}
 	close $in
 	close $out
-	frename $mhProfile(context).new $mhProfile(context)
+	file rename -force $mhProfile(context).new $mhProfile(context)
 	return $value
     } else {
 	close $in
@@ -425,17 +437,10 @@ proc MhReadSeqs {folder seqsvar} {
     # First read the private sequence
     set mhPriv(changed,private) 0
     set filename $mhProfile(context)
-    if [file exists $filename] {
-	set mtime [file mtime $filename]
+    if {![catch {set mtime [file mtime $filename]}]} {
 	if {![info exists mhPriv(privmtime)] || ($mtime != $mhPriv(privmtime))} {
-	    foreach elem [array names mhPriv] {
-		set indices [split $elem ,]
-		if {[lindex $indices 0] == {privseq}} {
-		    if {[lindex $indices 1] == $folder} {
-			unset mhPriv($elem)
-		    }
-		}
-	    }
+            array unset mhPriv privseq,${folder},*
+            FlistUncacheLocal $folder
 	    if {[catch {open $filename r} in] == 0} {
 		Exmh_Debug MhReadSeqs Reading $filename
 		set old [read $in]
@@ -458,29 +463,23 @@ proc MhReadSeqs {folder seqsvar} {
 		set mhPriv(privmtime) $mtime
 	    }
 	}
+    } elseif {[info exists mhPriv(privmtime)]} {
+	unset mhPriv(privmtime)
+        array unset mhPriv privseq,${folder},*
+        FlistUncacheLocal $folder
     }
-    foreach elem [array names mhPriv] {
+    # mhPriv(privseq,folder,sequence) contains list of message IDs
+    foreach elem [array names mhPriv privseq,${folder},*] {
 	set indices [split $elem ,]
-	if {[lindex $indices 0] == {privseq}} {
-	    if {[lindex $indices 1] == $folder} {
-		set seqs([lindex $indices 2]) $mhPriv($elem)
-	    }
-	}
+        set seqs([lindex $indices 2]) $mhPriv($elem)
     }
     # Then read the public sequence
     set mhPriv(changed,public) 0
     set filename "$mhProfile(path)/$folder/$mhProfile(mh-sequences)"
-    if [file exists $filename] {
-	set mtime [file mtime $filename]
+    if {![catch {set mtime [file mtime $filename]}]} {
 	if {![info exists mhPriv(seqmtime,$folder)] || ($mtime != $mhPriv(seqmtime,$folder))} {
-	    foreach elem [array names mhPriv] {
-		set indices [split $elem ,]
-		if {[lindex $indices 0] == {pubseq}} {
-		    if {[lindex $indices 1] == $folder} {
-			unset mhPriv($elem)
-		    }
-		}
-	    }
+            array unset mhPriv pubseq,${folder},*
+            FlistUncacheLocal $folder
 	    if {[catch {open $filename r} in] == 0} {
 		Exmh_Debug MhReadSeq Reading $filename
 		set old [read $in]
@@ -505,14 +504,14 @@ proc MhReadSeqs {folder seqsvar} {
 		set mhPriv(seqmtime,$folder) $mtime
 	    }
 	}
+    } elseif {[info exists mhPriv(seqmtime,$folder)]} {
+	unset mhPriv(seqmtime,$folder)
+        array unset mhPriv pubseq,${folder},*
+        FlistUncacheLocal $folder
     }
-    foreach elem [array names mhPriv] {
+    foreach elem [array names mhPriv pubseq,${folder},*] {
 	set indices [split $elem ,]
-	if {[lindex $indices 0] == {pubseq}} {
-	    if {[lindex $indices 1] == $folder} {
-		set seqs([lindex $indices 2]) $mhPriv($elem)
-	    }
-	}
+        set seqs([lindex $indices 2]) $mhPriv($elem)
     }
 }
 
@@ -590,17 +589,12 @@ proc Mh_SequenceUpdate { folder how seq {msgids {}} {which public}} {
     if {[info exist seqs]} {
       unset seqs
     }
-    foreach elem [array names mhPriv] {
-	set indices [split $elem ,]
-	if {[lindex $indices 0] == {mode}} {
-	    if {[lindex $indices 1] == $folder} {
-		unset mhPriv($elem)
-	    }
-	}
-    }
+    array unset mhPriv(mode,$folder)    ;# array unset is ok if already unset
     MhReadSeqs $folder seqs
     # Set the value for the sequence we're updating
-    if [catch {set seqs($seq)} oldmsgids] {
+    if {[info exist seqs($seq)]} {
+        set oldmsgids $seqs($seq)
+    } else {
 	set oldmsgids {}
     }
     set seqs($seq) [MhSeq $folder $seq $how $oldmsgids $msgids]
@@ -615,6 +609,7 @@ proc Mh_SequenceUpdate { folder how seq {msgids {}} {which public}} {
     }
     if {$mhPriv(changed,public) == 1} {
 	set mhPriv(pubseq,$folder,$seq) [MhSeqExpand $folder $seqs($seq)]
+        FlistUncacheLocal $folder
 	set filename $mhProfile(path)/$folder/$mhProfile(mh-sequences)
 	if {[catch {open $filename.new w} out] == 0} {
 	    Exmh_Debug Writing $filename
@@ -646,6 +641,7 @@ proc Mh_SequenceUpdate { folder how seq {msgids {}} {which public}} {
     }
     if {$mhPriv(changed,private) == 1} {
 	set mhPriv(privseq,$folder,$seq) [MhSeqExpand $folder $seqs($seq)]
+        FlistUncacheLocal $folder
 	set filename $mhProfile(context)
 	if {[catch {open $filename.new w} out] == 0} {
 	    Exmh_Debug Writing $filename
@@ -797,7 +793,7 @@ proc Mh_Rmm { folder msgids } {
 	eval {MhExec rmm +$folder} $chunk
     }
 }
-proc Mh_Send { msgid } {
+proc Mh_Send { msgid argu} {
     global mhProfile
     
     set path $mhProfile(path)/$mhProfile(draft-folder)/$msgid
@@ -806,16 +802,16 @@ proc Mh_Send { msgid } {
     switch -- $mhProfile(sendType) {
 	"async" {
 	    MhExec $mhProfile(sendproc) -draftf +$mhProfile(draft-folder) \
-		-draftm $dst -push -forward < /dev/null
+		-draftm $dst $argu -push -forward < /dev/null
 	}
 	"wait" {
 	    MhExec $mhProfile(sendproc) -draftf +$mhProfile(draft-folder) \
-		-draftm $dst < /dev/null
+		-draftm $dst $argu < /dev/null
 	}
 	"xterm" {
 	    eval exec $mhProfile(xtermcmd) { \
 		-title "Sending $mhProfile(draft-folder)/$msgid ..." \
-		-e sh -c "$mhProfile(sendproc) -draftf +$mhProfile(draft-folder) -draftm $dst || whatnow -draftf +$mhProfile(draft-folder) -draftm $dst" &}
+		-e sh -c "$mhProfile(sendproc) -draftf +$mhProfile(draft-folder) -draftm $dst $argu || whatnow -draftf +$mhProfile(draft-folder) -draftm $dst" &}
 	}
     }
     if {$msgid != $dst} {
@@ -931,7 +927,7 @@ proc MhParseProfile {} {
 	set mhProfile(draft-folder) [string trim $mhProfile(draft-folder) +]
 	if {![file isdirectory $mhProfile(path)/$mhProfile(draft-folder)]} {
 	    Exmh_Status "Creating drafts folder"
-	    if {[catch {exec mkdir $mhProfile(path)/$mhProfile(draft-folder)} msgid]} {
+	    if {[catch {file mkdir $mhProfile(path)/$mhProfile(draft-folder)} msgid]} {
 		catch {
 		    puts stderr "Cannot create drafts folder $mhProfile(path)/$mhProfile(draft-folder)"
 		}
@@ -1003,7 +999,7 @@ It appears you have not used the MH mail system before.
 Normally MH creates a directory named ~/Mail and puts
 its mail folders and some other files under there.
 If you want your folders elsewhere, you will have to
-exit Exmh and run the program install-mh by hand.
+exit Exmh and run the program /usr/bin/mh/install-mh by hand.
 
 Is it ok if Exmh sets up your MH environment for you?
 "
@@ -1019,10 +1015,10 @@ Is it ok if Exmh sets up your MH environment for you?
 proc MhSetupNewUserInner {} {
     global mhProfile exmh
     set exmh(newuser) 1
-    catch {exec mkdir [glob ~]/Mail}
+    catch {file mkdir [glob ~]/Mail}
     if {![file exists $mhProfile(profile)]} {
 	set out [open $mhProfile(profile) w]
-	puts $out "Path: Mail"
+	puts $out "Path: Mail\nMsg-Protect: 600\nFolder-Protect: 700"
 	close $out
     }
     catch {MhExec inc < /dev/null} result
@@ -1063,7 +1059,7 @@ proc MhSetupDraftFolderInner {} {
     set dir $mhProfile(path)/$mhProfile(draft-folder)
     if {![file isdirectory $dir]} {
 	if {[catch {
-	    exec mkdir $dir
+	    file mkdir $dir
 	    Exmh_Status "Created drafts folder \"+drafts\""
 	} err]} {
 	    Exmh_Status "Cannot create a drafts folder! $err" error
@@ -1190,14 +1186,8 @@ proc Mh_Pathname { profile } {
     return $mhProfile(path)/$profile
 }
 
-set mh_mv_flag -f
 proc Mh_Rename { old new } {
-    global mh_mv_flag tk_version
-    if {$tk_version >= 4.2} {
 	file rename -force $old $new
-    } else {
-	eval exec mv $mh_mv_flag {$old $new} < /dev/null
-    }
 }
 
 # find a *comp* file going up from the current folder
