@@ -13,31 +13,23 @@
 # any specification.
 
 proc Mime_Init {} {
-    global mime env base64 mimeFont
+    global install mime env base64 mimeFont
 
     if [info exists mime(init)] {
 	return
     }
-    # Make sure Metamail is on the path
-    set hit 0
+
+    # no more metamail required
+    set env(NOMETAMAIL) 1
+
+    # look for mimencode or recode in path, prefer mimencode
+    set rhit 0
     foreach dir [split $env(PATH) :] {
-	if {[string compare $dir $mime(dir)] == 0} {
-	    set hit 1
-	    break
+	if {[file executable "$dir/mimencode"]} {
+	    set mime(encode) "$dir/mimencode"
 	}
-    }
-    if {! $hit} {
-	set env(PATH) $mime(dir):$env(PATH)
-    }
-    set mime(encode) mimencode
-    foreach dir [split $env(PATH) :] {
-	if {[file executable $dir/mimencode]} {
-	    set mime(encode) mimencode
-	    break
-	}
-	if {[file executable $dir/mmencode]} {
-	    set mime(encode) mmencode
-	    break
+	if {[file executable "$dir/recode"]} {
+	    set mime(recode) "$dir/recode"
 	}
     }
 
@@ -118,9 +110,13 @@ and the official start of a multipart message is displayed.  Sometimes
 this has useful information, while other times it has warnings about
 the rest of the message being in MIME format."}
 	{mime(fullHeaders) mimeFullHeaders	OFF {Show full headers}
-"This controls whether full headers are shown for message/rfc822 items
-inside MIME mail.  This prevents Folder-Display and Folder-Suppress
-profile options from taking effect."}
+"This controls whether full headers are shown for MIME mails. If enabled,  
+Header-Display and Header-Suppress .mh_profile options are ignored and 
+all headers are shown.
+If disabled, these profile options are live. Default action is to hide 
+headers (scrolled away). Also, tagnames \"hidden\" and \"default\" are 
+available for colorizing hidden and shown headers, respectively.
+See \"Colorizing Headers\" in the Customization document."}
 	{mime(maxSubpartsDisplayed) mimeMaxSubpartsDisplayed	5 
 	    {Maximum subparts to display}
 "This is the maximum number of subparts to display without hiding complex
@@ -131,16 +127,15 @@ may be displayed by hand from the menu."}
 "This is the maximum size of a message before exmh displays a
 warning about a large message and STOP button."}
 	{mime(ftpMethod) ftpMethod
-        {CHOICE expect ftp {ftp -n} metamail {URI tool}}
+        {CHOICE expect ftp {ftp -n} {URI tool}}
 	{FTP access method}
 "Sometimes the automatic FTP transfer fails because of
 problems logging into the remote host.	This option lets
-you try a few different approachs to find the one that
+you try a few different approaches to find the one that
 works for you:
 expect - use the ftp.expect script to download the file.
 ftp - use ftp and feed user and password to it via a pipe.
 ftp -n - use the ftp no-auto-login feature.
-metamail - pass control to metamail and let it try.
 URI tool - uses your favorite WWW browser to get the
            file.  See \"URI Preferences\"."}
 	{mime(ftpCommand) ftpCommand	ftp {FTP command name}
@@ -184,7 +179,7 @@ The full header is: Content-Disposition: attachment; filename=\"...\"."}
 "This controls whether or not attachement filename
 will be translated into a suitable DOS 8.3 representation."}
 	{mime(mdnTo) dispositionNotificationTo {} {Disposition Notification To}
-"The address where you want MDN receipts to end up.  This normally your
+"The address where you want MDN receipts to end up.  This is normally your
 own email address."}
 	{mime(mdnSend) dispositionNotificationSend
          {CHOICE never deny {ask user} auto/ask auto/ignore } 
@@ -211,6 +206,10 @@ The following line makes exmh display the text/plain part in favour
 of the text/html part if both are present in a multipart/alternative:
 
 text/plain text/html"}
+{mime(saveDir) mimeSaveDir {~/attachments} {Save directory for attachments}
+"This defines the directory to use when saving attachments.  Attachments
+are saved by extracting them from the message and replacing the
+attachment with a message/external-body reference to the saved file."}
     }
     set i 0
     foreach char {A B C D E F G H I J K L M N O P Q R S T U V W X Y Z \
@@ -312,12 +311,20 @@ proc MimeHeader {part contentType encoding} {
     if {[string compare $type "text"] == 0} {
 	set type text/plain
     }
+    # Paranoia time - sanitize these just in case...
+    # rfc2045: any ascii char except space, controls and
+    # the tspecials: ()<>@,;:\"/[]?=
+    # the forward slash is kept.
     set mimeHdr($part,hdr,content-type) $contentType
+    regsub -all {[[:cntrl:][:blank:]()<>@,;:\\"?=[.[.][.].]]} $type {} type 
+    # "
     set mimeHdr($part,type) $type
+    regsub -all {[^-[:print:]]} $encoding {} encoding
     set mimeHdr($part,encoding) $encoding
     set mimeHdr($part,params) {}
     foreach sub [lrange $params 1 end] {
 	if [regexp {([^=]+)=(.+)} $sub match key val] {
+	    regsub -all {[^-/[:print:]]} $key {} key
 	    set key [string trim [string tolower $key]]
 	    set val [string trim $val]
 	    # Allow single as well as double quotes
@@ -336,9 +343,8 @@ proc MimeHeader {part contentType encoding} {
 	set mimeHdr($part,param,charset) \
 		[string tolower $mimeHdr($part,hdr,x-sun-charset)]
     }
-    # infer the name from the filename:
-    if {[info exists mimeHdr($part,param,filename)] &&
-      ![info exists mimeHdr($part,param,name)]} {
+    # infer the name from the filename= in preference to a name=
+    if {[info exists mimeHdr($part,param,filename)]} {
       set mimeHdr($part,param,name) $mimeHdr($part,param,filename)
     }
     return $type
@@ -617,11 +623,20 @@ proc MimeClose { fileIO } {
 proc MimeSetDisplayFlag {part} {
     global mime mimeHdr msg uri
 
-    # Flag to determine if we display the part or not.	We display it
+    # Flag to determine if we display the part or not. We don't display it
+    # if we've been told not to (yes, we check this first, as RFC2183, sec 2.9
+    # says it's legal even on a multipart). Otherwise, we display it
     # if it's a multipart (other than multipart/parallel).  We display 
     # it if it's an image (unless we aren't supposed to do that.  Otherwise,
-    # we display it unless we've been told not to or it's too darn big.
+    # we display it unless it's too darn big.
     if {[info exists mimeHdr($part,file)]} {
+	if {[info exists mimeHdr($part,hdr,content-disposition)]} {
+	    set disp [split $mimeHdr($part,hdr,content-disposition) \;]
+	    if [regexp {ancillary|attachment} [lindex $disp 0]] {
+		set mimeHdr($part,display) 0
+		return
+	    }
+	}
 	if [regexp {^multipart} $mimeHdr($part,type)] {
 	    set mimeHdr($part,display) \
 		[expr ![regexp {parallel$} $mimeHdr($part,type)]]
@@ -638,9 +653,11 @@ proc MimeSetDisplayFlag {part} {
 	    Exmh_Debug "vcard: not displaying"
 	    return
 	}
-	if {[info exists mimeHdr($part,hdr,content-disposition)]} {
-	    set disp [split $mimeHdr($part,hdr,content-disposition) \;]
-	    if [regexp {ancillary|attachment} [lindex $disp 0]] {
+	# Special case to not show the original message in a SPAM
+	# wrapped report. 
+	if {[info exists mimeHdr($part,hdr,content-description)]} {
+	    set disp [split $mimeHdr($part,hdr,content-description) \;]
+	    if [regexp {original message before SpamAssassin} [lindex $disp 0]] {
 		set mimeHdr($part,display) 0
 		return
 	    }
@@ -1432,23 +1449,53 @@ proc MimeDecode {fileName name encoding text} {
     }
     if [catch {
 	set out [open $name w 0600]
+        fconfigure $out -encoding binary        ;# No CRLF translations, please
 	switch -regexp -- $encoding {
 	    (8|7)bit {
 		Exmh_Debug "cat > $name"
 		exec cat $fileName >@ $out
 	    }
+	    quoted-printable -
 	    base64 {
-		if $text {
-		    Exmh_Debug "$mime(encode) -u -b -p > $name"
-		    exec $mime(encode) -u -b -p $fileName >@ $out
-		} else {
-		    Exmh_Debug "$mime(encode) -u -b > $name"
-		    exec $mime(encode) -u -b $fileName >@ $out
-		}
-	    }
-	    quoted-printable {
-		Exmh_Debug "$mime(encode) -u -q > $name"
-		exec $mime(encode) -u -q $fileName >@ $out
+		# tre mimencode or recode if available
+                if {[info exist mime(encode)] || [info exists mime(recode)]} {
+		    if {[info exist mime(encode)]} {
+			set cmd [list exec $mime(encode) -u]
+			lappend cmd [expr {$encoding == "base64"?"-b":"-q"}]
+			lappend cmd $fileName >@ $out
+		    } else {
+			set cmd [list exec $mime(recode)]
+			lappend cmd [expr {$encoding == "base64"?"base64..data":"qp..data"}]
+			lappend cmd < $fileName >@ $out
+		    }
+		    eval $cmd
+                } else {
+                  # Replace use of mimencode with Tcl versions.
+                  # This is noticably slower.  We should also
+                  # figure out how to use mhstore here
+                  set in [open $fileName]
+                  set line_cnt 0  ;# Accumulate lines before calling the decoder
+                  set buffer ""
+                  while {[gets $in line] >= 0} {
+                      append buffer $line\n
+                      incr line_cnt
+                      if {$line_cnt > 1000} {
+                          if {$encoding == "base64"} {
+                              puts -nonewline $out [Base64_Decode $buffer]
+                          } else {
+                              puts -nonewline $out [::mime::qp_decode $buffer]
+                          }
+                          set line_cnt 0
+                          set buffer ""
+                      }
+                  }
+                  if {$encoding == "base64"} {
+                      puts -nonewline $out [Base64_Decode $buffer]
+                  } else {
+                      puts -nonewline $out [::mime::qp_decode $buffer]
+                  }
+		  catch  {close $in}
+                }
 	    }
 	    .*uue.* {
 		Exmh_Debug "uudecode -p > $name"
@@ -1561,7 +1608,7 @@ proc MimeLocalFileTransfer {tkw part} {
     set mimeHdr($part=1,file) [file join $dir $name]
 }
 proc MimeFTPTransfer {tkw part} {
-    global mime mimeHdr
+    global mime mimeHdr install
 
     set site $mimeHdr($part,param,site)
     set directory $mimeHdr($part,param,directory)
@@ -1584,7 +1631,7 @@ proc MimeFTPTransfer {tkw part} {
 	    case $mime(ftpMethod) {
 		expect {
 		    Exmh_Status "ftp.expect $site ..."
-		    busy exec ftp.expect $site $directory $theirname $myname $mode
+		    busy exec $install(dir,bin)/ftp.expect $site $directory $theirname $myname $mode
 		}
 		ftp* {
 		    Exmh_Status "$mime(ftpCommand) -n $site ..."
@@ -2335,11 +2382,21 @@ proc Mime_SavePiece {part type} {
 	return
     }
     Exmh_Status "Saving $type $fileName"
-    set name [FSBox "Save $type to:" $default]
+    set name [FSBox "Save $type to:" $default write]
     if {$name != {}} {
+	set forceit ""
+	if [file exists $name ] {
+	    if {! [FileExistsDialog $name]} {
+		Exmh_Status "Save canceled"
+		return 0
+	    }
+	    set forceit "-force"
+	}
 	if [catch {
-	    exec cp $fileName $name
-	    exec chmod $mhProfile(msg-protect) $name
+	    eval file copy $forceit -- $fileName $name
+            # Leading zero to ensure octal interpretation
+	    file attributes $name -permissions 0$mhProfile(msg-protect)
+	    Exmh_Status "Saved to $name"
 	} err] {
 	    Exmh_Status $err
 	}
@@ -2816,14 +2873,7 @@ proc Mime_EudoraFilename {name} {
 }
 
 proc Mime_MakeBoundary { end } {
-
-    # Old boundary was made by
-    # regsub -all "\[ \x7f-\xff\]" [exec date] _ date
-    # This way works at least with GNU date.
-
-    regsub -all {[^0-9]|[0-9][0-9][0-9][0-9]} [exec date] {} boundary
-    regsub {^0*} $boundary {} boundary	;# No octal interpretation, please
-    set boundary [expr $boundary*[pid]]
+    set boundary [clock seconds]_[pid]
     return "==_Exmh_$boundary$end"
 }
 proc Mime_Magic { fileName } {
@@ -2872,3 +2922,42 @@ proc Mime_ShowXApp { tkw part } {
      [list Mime_SavePiece $part $mimeHdr($part,type)]
     $tkw insert insert \n
 }
+
+proc Mime_SaveAttachments {{dir {}}} {
+    global msg mime install argv0
+
+    if {$dir == ""} {
+      set dir $mime(saveDir)
+    }
+    file mkdir $dir
+    if {[catch {
+      exec ${argv0}-strip $msg(path) $dir
+    } err]} {
+        if {[regexp {(.*) exists} $err _ file]} {
+            if {[Mime_SaveAttachmentRetry $file]} {
+                file delete $file
+                exec ${argv0}-strip $msg(path) $dir
+            }
+        }
+    }
+    Msg_ShowCurrent
+}
+proc Mime_SaveAttachmentRetry {file} {
+    global exwin mime
+    set f [frame $exwin(mtext).save_err -bd 2 -relief ridge]
+
+    message $f.msg1 -text "File Exists\n$file\nDelete and retry?" -aspect 1000
+    pack $f.msg1 -side top -fill both
+
+    set b1 [frame $f.but -bd 10 -relief flat]
+    pack $b1 -side top
+
+    set mime(retry_save) 0
+    button $b1.cancel -text "Cancel" -command {destroy $exwin(mtext).save_err}
+    button $b1.ok -text "OK" -command {set mime(retry_save) 1; destroy $exwin(mtext).save_err}
+    pack $b1.cancel $b1.ok -side left -padx 3
+
+    Widget_PlaceDialog $exwin(mtext) $f
+    tkwait window $f
+    return $mime(retry_save)
+}
