@@ -64,6 +64,9 @@ proc Msg_Show { {seq cur} {show show} } {
 proc Msg_ShowCurrent {} {
     Msg_Show cur
 }
+proc Msg_ShowUnseen {} {
+    Msg_Show unseen
+}
 proc Msg_ClearCurrent { } {
     global msg exmh
     set msg(id) {}		;# Clear current message
@@ -115,11 +118,11 @@ proc MsgChange {msgid {show show}} {
     Ftoc_ClearCurrent
     set oldcur [Seq_Msgs $exmh(folder) cur]
     Mh_SetCur $exmh(folder) $msgid
-    Ftoc_ShowSequence cur [concat $oldcur $msgid]
     set lineno [Ftoc_FindMsg $msgid]
     if {! [Ftoc_Change $lineno $show]} {
 	Exmh_Status "Cannot find msg $msgid - Rescan?"
     } else {
+	Ftoc_ShowSequence cur [concat $oldcur $msgid]
 	if {$msg(id) == {}} {
 	    Buttons_Current 1
 	}
@@ -554,7 +557,7 @@ proc Msg_Save {} {
 	lappend files $mhProfile(path)/$exmh(folder)/$msgid
     }
     
-    set name [FSBox "Select file to create/append to:" ]
+    set name [FSBox "Select file to create/append to:" "" write]
     if {$name != {}} {
 	set exists [file exists $name]
 	if [catch {eval {exec cat} $files {>> $name}} err] {
@@ -603,7 +606,7 @@ proc Msg_Edit {} {
 
 proc Msg_UUdecode {} {
     global msg
-    set name [FSBox "Select file to decode into:" ]
+    set name [FSBox "Select file to decode into:" "" write]
     if {$name != {}} {
 	Mime_Uudecode $msg(path) $name
     } else {
@@ -684,3 +687,9 @@ proc MsgDecodeURL { url } {
     return $url
 }
 
+# exmh-2.5 APIs
+# Msg_CheckPoint
+# Msg_MarkUnseen
+# Msg_Seen
+# Msg_ShowWhat
+# Msg_UnSeen
