@@ -6,6 +6,54 @@
 # 
 
 # $Log: pgpExec.tcl,v $
+# Revision 1.22  2008/06/18 10:06:15  az143
+# patch from debian: added support for gnupg's gpg-agent
+#
+# Revision 1.21  2005/01/01 20:16:20  welch
+# Based on patches from Alexander Zangerl
+# lib/pgpGpg.tcl:
+# lib/pgpPgp5.tcl:
+#   fix of an old pgp problem where recipients were duplicated when
+#   pgp is run in interactive mode
+# lib/extrasInit.tcl: a small documentation improvement for the
+#   pgp(getextcmd) functionality.
+#   faces(xfaceProg) gains a default (uncompface -X)
+# lib/pgpExec.tcl: fix for http://bugs.debian.org/164210: multiple gpg
+#   subkeys and passphrases.  exmh would not ask for the right passphrase.
+# lib/addr.tcl: ldap options gain defaults that are compatible with
+#   debian's openldap config
+# lib/mh.tcl: add Msg-Protect and Folder-Protect to the default .mh_profile
+#   that is generated when setting up new users.
+#
+# (These changes are inspired by a patch from Alexander, but not the same)
+# lib/inc.tcl: use $install(dir,bin) to specify an absolute path to
+#   the inc.expect script
+# lib/seditExtras.tcl: use $install(dir,bin) to specify an absolute path to
+#   the exmh-async script
+# lib/mime.tcl: use $install(dir,bin) to specify an absolute path to
+#   the ftp.expect script.
+#   Also changed MimeMakeBoundary to use [clock seconds] instead of
+#   re-writing the output of [exec date].
+#
+# Revision 1.20  2003/02/18 06:50:43  welch
+#     extrasInit.tcl, pgp.tcl - picked up pgp(extpass) patch from Alexander Zangerl
+#     flist.tcl - FlistFindSeqsInner added check to eliminate calls to
+#         Seq_Set if the sequence information for a folder hasn't changed
+#     ftoc.tcl - Changed msgtolinecache and linetomsgcache so that
+#         they have no entries for empty ({}) mappings.  I was running into
+#         a mapping for the last text widget line that didn't contain a message
+#         and ended up messing up incremental folder scans
+#         Ftoc_MsgNumber doesn't cache anything if there is no mapping
+#         Retrieved FtocShowUnseen from exmh-2.5 and use that for the
+#         unseen sequence instead of the more general search
+#     main.tcl - a slight varition on the fix that slipcon made to the
+#         millisecond time stamps.
+#     mh.tcl - rooted out an "array unset" that doesn't work in Tcl 8.0
+#     thread.tcl - fixed call to Flist_ForgetSequence (changed to Seq_Forget)
+#     Minor HTML cleanup, including pointer to the Wiki.
+#     I added comments to several files that identify old exmh APIs, including
+#     flist.tcl, folder.tcl, mh.tcl, msg.tcl,
+#
 # Revision 1.19  2002/07/16 01:27:54  sysphrog
 # Fixing problems with PGP sign+encrypt, gnupg 1.0.7 support
 #
@@ -168,27 +216,33 @@ proc Pgp_Exec { v exectype arglist outvar {privatekey {}} {interactive 0} } {
     if {![set pgp(keeppass)]} {
 	Pgp_ClearPassword $v
     }
-    if {$interactive || !([set pgp(keeppass)] || ($privatekey == {}))} {
-        Exmh_Debug "<Pgp_Exec> Pgp_Exec_Interactive $v $exectype $arglist output"
-	return [Pgp_Exec_Interactive $v $exectype $arglist output]
+    # gnupg agent requested? then batch!
+    if {[set pgp(gpg,useagent)]} {
+	Exmh_Debug "<Pgp_Exec> Pgp_Exec_Batch $v $exectype $arglist output"
+	return [Pgp_Exec_Batch $v $exectype $arglist output]
     } else {
-	if {$privatekey == {}} {
-            Exmh_Debug "<PGP Pgp_Exec> Pgp_Exec_Batch $v $exectype $arglist output"
-	    return [Pgp_Exec_Batch $v $exectype $arglist output]
+	if {$interactive || !([set pgp(keeppass)] || ($privatekey == {}))} {
+	    Exmh_Debug "<Pgp_Exec> Pgp_Exec_Interactive $v $exectype $arglist output"
+	    return [Pgp_Exec_Interactive $v $exectype $arglist output]
 	} else {
-	    Exmh_Debug v=$v
-
-	    set keyid [lindex $privatekey 0]
-	    Exmh_Debug keyid=$keyid
-	    # Check for passphrase. Pgp_GetPass is cache and expire aware!
-	    set p [Pgp_GetPass $v $privatekey]
-	    #Exmh_Debug "<Pgp_Exec> got passwd >$p<"
-
-	    if {[string length $p] == 0} {
-		return 0
+	    if {$privatekey == {}} {
+		Exmh_Debug "<Pgp_Exec> Pgp_Exec_Batch $v $exectype $arglist output"
+		return [Pgp_Exec_Batch $v $exectype $arglist output]
+	    } else {
+		Exmh_Debug v=$v
+		
+		set keyid [lindex $privatekey 0]
+		Exmh_Debug keyid=$keyid
+		# Check for passphrase. Pgp_GetPass is cache and expire aware!
+		set p [Pgp_GetPass $v $privatekey]
+		#Exmh_Debug "<Pgp_Exec> got passwd >$p<"
+		
+		if {[string length $p] == 0} {
+		    return 0
+		}
+		Exmh_Debug "<Pgp_Exec> Pgp_Exec_Batch $v $exectype $arglist output \(password\)"
+		return [Pgp_Exec_Batch $v $exectype $arglist output $p]
 	    }
-            Exmh_Debug "<Pgp_Exec> Pgp_Exec_Batch $v $exectype $arglist output \(password\)"
-	    return [Pgp_Exec_Batch $v $exectype $arglist output $p]
 	}
     }
 }
@@ -526,13 +580,15 @@ proc Pgp_Exec_GetDecryptKey {v in recipients} {
         set key [list SYM {} {} {} "symmetrically encrypted message"]
       } else {
 	  # One of user's private keys?  If so, than use it.
+	  # make sure that we look at *all* subkeys
         foreach key [set pgp($v,privatekeys)] {
-          if {[regexp $keyid [lindex $key 0]]} {
-            return $key
-          } elseif {[regexp $keyid [lindex $key 2]]} {
+	    for {set i 0} {$i<[expr [llength $key]-1]} {incr i 2} {
+		if {[regexp $keyid [lindex $key $i]]} {
             return $key
           }
         }
+	}
+	return {}
       }
     } else {
       set recipients [string tolower $recipients]
@@ -689,6 +745,33 @@ proc Pgp_Exec_ExtractKeys { v file outvar {interactive 1} } {
 proc Pgp_GetPass { v key } {
     global pgp
 
+    if {[info exists pgp(extpass)] && [set pgp(extpass)] \
+	    && [info exists pgp(getextcmd)]} {
+	Exmh_Debug "Pgp_GetPass $v $key external"
+	set keyid [lindex $key 0]
+	set cmd [format $pgp(getextcmd) $keyid]
+	while (1) {
+	    Exmh_Debug "running cmd $cmd"
+	    if [ catch {exec sh -c "$cmd"} result ] {
+		Exmh_Debug "error running cmd: $result"
+		Exmh_Status "Error executing external cmd" warn
+		return {}
+	    } else {
+		if {[Pgp_Exec_CheckPassword $v $result $key]} {
+		    return $result
+		} else {
+		    Exmh_Debug "bad passphrase"
+		    if {[info exists pgp(delextcmd)]} {
+			Exmh_Debug "trying to invalidate bad passphrase"
+			if [catch {exec sh -c "[format $pgp(delextcmd) $keyid]"}] {
+			    Exmh_Debug "invalidation failed"
+			    return {}
+			}
+		    }
+		}
+	    }
+	}
+    } else {
     Exmh_Debug "Pgp_GetPass $v $key"
 
     if {[lsearch -glob [set pgp($v,privatekeys)] "[lindex $key 0]*"] < 0} {
@@ -737,6 +820,7 @@ proc Pgp_GetPass { v key } {
             }
             return $password
         }
+	}
     }
 }
 
