@@ -1,6 +1,43 @@
 # pgpGpg.tcl
 
 # $Log: pgpGpg.tcl,v $
+# Revision 1.16  2012/01/22 22:54:45  az143
+# az@debian.org repaired gpg cipher option handling
+#
+# Revision 1.15  2008/06/18 10:06:15  az143
+# patch from debian: added support for gnupg's gpg-agent
+#
+# Revision 1.14  2005/01/01 20:16:20  welch
+# Based on patches from Alexander Zangerl
+# lib/pgpGpg.tcl:
+# lib/pgpPgp5.tcl:
+#   fix of an old pgp problem where recipients were duplicated when
+#   pgp is run in interactive mode
+# lib/extrasInit.tcl: a small documentation improvement for the
+#   pgp(getextcmd) functionality.
+#   faces(xfaceProg) gains a default (uncompface -X)
+# lib/pgpExec.tcl: fix for http://bugs.debian.org/164210: multiple gpg
+#   subkeys and passphrases.  exmh would not ask for the right passphrase.
+# lib/addr.tcl: ldap options gain defaults that are compatible with
+#   debian's openldap config
+# lib/mh.tcl: add Msg-Protect and Folder-Protect to the default .mh_profile
+#   that is generated when setting up new users.
+#
+# (These changes are inspired by a patch from Alexander, but not the same)
+# lib/inc.tcl: use $install(dir,bin) to specify an absolute path to
+#   the inc.expect script
+# lib/seditExtras.tcl: use $install(dir,bin) to specify an absolute path to
+#   the exmh-async script
+# lib/mime.tcl: use $install(dir,bin) to specify an absolute path to
+#   the ftp.expect script.
+#   Also changed MimeMakeBoundary to use [clock seconds] instead of
+#   re-writing the output of [exec date].
+#
+# Revision 1.13  2001/12/06 16:39:13  kchrist
+# Exmh can now parse the GnuPG options file and identify the
+# "default-key" (same as "myname" in PGP).  Added "--status-fd 2" to
+# args_decrypt so that the output can be parsed with Pgp_InterpretOutput.
+#
 # Revision 1.12  2000/06/15 17:03:11  valdis
 # Add X-Mailer: change, fix PGP Comment: line...
 #
@@ -90,7 +127,6 @@ set pgp(pref,HKPkeyserverUrl) { HKPkeyserverUrl HKPKeyServerUrl {keys.pgp.com}
 It's used to tranfer keys to and from a keyserver.
 Give here a hkp server name." }
 
-
 # Needed for Preferences
 set pgp(gpg,description) "GNUPG is a free GPLed PGP clone written by Werner Koch"
 set pgp(gpg,prefs) [list rfc822 \
@@ -105,6 +141,10 @@ proc Pgp_gpg_Preferences {} {
     # GnuPG algorithms and algorithm modules
     set label $pgp(gpg,fullName)
     Preferences_Add "$label interface" {} [list \
+       [list pgp(gpg,useagent) gpgUseAgent OFF "Use GnuPG Agent" \
+"If this option is enabled, then Exmh defers all passphrase 
+handling to GnuPG's gpg-agent. This option overrides and disables 
+the pgpKeepPass option." ] \
                 [list pgp(gpg,comment) gpgComment \
 "Exmh [set exmh(version)]" "GnuPG Comment" \
 "Specify the comment GnuPG should put in the comment field
@@ -133,6 +173,11 @@ ON "PGP 5.0 Compatibility" \
 PGP 5.0 compatible messages.
 Having this enabled, you don't need PGP 5.0 any more." ] ]
 
+### clear keeppass if agent is on
+if {[set pgp(gpg,useagent)]} {
+    set pgp(keeppass) 0
+}
+
     # Before we can build the algorithm choice preferences part
     # we need to examine, which modules are installed on the system
     # and build a complete list of algos
@@ -142,25 +187,23 @@ Having this enabled, you don't need PGP 5.0 any more." ] ]
     Preferences_Add "$label interface" {} [list \
                 [list pgp(gpg,cipheralgo) gpgCipherAlgo \
 [concat CHOICE $pgp(gpg,cipheralgos)] "Default Cipher Algo" \
-"Your preferred cipher algorithm."] \
+"Your preferred cipher algorithm. Select 'default' to leave the choice to GnuPG."] \
                 [list pgp(gpg,digestalgo) gpgDigestAlgo \
 [concat CHOICE $pgp(gpg,digestalgos)] "Default Digest Algo" \
-"Your preferred digest algorithm."] \
+"Your preferred digest algorithm. Select 'default' to leave the choice to GnuPG."] \
                 [list pgp(gpg,compressalgo) gpgCompressAlgo \
 [concat CHOICE $pgp(gpg,compressalgos)] "Default Compress Algo" \
 "The algorithm, GnuPG uses to compress the text before encrypting.
-You have the choice between the ZIP (RFC1951)
-and the ZLIB (RFC1950) algo. ZIP is used by PGP(2/5).
-If you choose none, the text is left uncompressed." ] ]
+Select 'default' to leave the choice to GnuPG, or 'none' to leave text uncompressed." ] ]
     } 
 
 
 #######################################################################
 # GPG BASIC CONFIG
 # builtin gpg algos
-set pgp(gpg,cipheralgos) {3des cast5 blowfish twofish}
-set pgp(gpg,digestalgos) {sha1 md5 ripemd160}
-set pgp(gpg,compressalgos) {zip zlib none}
+set pgp(gpg,cipheralgos) {default cast5 idea 3des blowfish aes aes192 aes256 twofish camellia128 camellia192 camellia256}
+set pgp(gpg,digestalgos) {default md5 sha1 ripemd160 sha224 sha256 sha384 sha512}
+set pgp(gpg,compressalgos) {default zip zlib bzip2 none}
 set pgp(gpg,pubkeyalgos) {}
 # module files
 set pgp(gpg,ciphermodfiles) {}
@@ -252,6 +295,9 @@ proc Pgp_Gpg_Arglist {} {
                  $pgp(gpg,pubkeymodfiles) ]
     ldelete modfiles {}
     set arglist [list --no-greeting --comment $pgp(gpg,comment)]
+    if {$pgp(gpg,useagent)} {
+	lappend arglist --use-agent
+    }
     # Take it
     if {$pgp(gpg,pgp5compatibility)} {
         lappend arglist --force-v3-sigs
@@ -277,14 +323,19 @@ proc Pgp_Gpg_Arglist {} {
         set arglist [concat $arglist [list \
                      --load-extension $modfile] ]
     }
-    set arglist [concat $arglist [list \
-                     --cipher-algo $pgp(gpg,cipheralgo) \
-                     --digest-algo $pgp(gpg,digestalgo) ] ]
+    if {$pgp(gpg,cipheralgo) != "default"} {
+	lappend arglist --personal-cipher-preferences $pgp(gpg,cipheralgo)
+    }
+    if {$pgp(gpg,digestalgo) != "default"} {
+	lappend arglist --personal-digest-preferences $pgp(gpg,digestalgo)
+    }
     # compressalgo
-    switch $pgp(gpg,compressalgo) {
-        zip  { set arglist [concat $arglist --compress-algo 1] }
-        zlib { set arglist [concat $arglist --compress-algo 2] }
-        none { set arglist [concat $arglist -z 0] }
+    if {$pgp(gpg,compressalgo) != "default" } {
+	if {$pgp(gpg,compressalgo) == "none"} {
+	    lappend arglist -z 0
+	} else {
+	    lappend arglist --personal-compress-preferences $pgp(gpg,compressalgo)
+	} 
     }
     ldelete arglist {}
     return $arglist
@@ -296,7 +347,7 @@ proc Pgp_Gpg_Arglist {} {
 #
 
 # Should config file be parsed
-set pgp(gpg,parse_config) 0
+set pgp(gpg,parse_config) 1
 
 #######
 # Exec
@@ -369,11 +420,11 @@ set pgp(gpg,args_exportKey) {--export --armor --textmode -o $file $keyid}
 
 ###############
 # Exec_Encrypt
-set pgp(gpg,args_encrypt) {[concat -eat -o $out [foreach id [Pgp_Misc_Map key {lindex $key 0} $tokeys] {lappend recips -r $id}; set recips] $in]}
+set pgp(gpg,args_encrypt) {[concat -eat -o $out [set recips {}; foreach id [Pgp_Misc_Map key {lindex $key 0} $tokeys] {lappend recips -r $id}; set recips] $in]}
 
 ###################
 # Exec_EncryptSign
-set pgp(gpg,args_encryptSign) {[concat -east -o $out -u $keyid [foreach id [Pgp_Misc_Map key {lindex $key 0} $tokeys] {lappend recips -r $id}; set recips] $in]}
+set pgp(gpg,args_encryptSign) {[concat -east -o $out -u $keyid [set recips {}; foreach id [Pgp_Misc_Map key {lindex $key 0} $tokeys] {lappend recips -r $id}; set recips] $in]}
 
 ############
 # Exec_Sign
@@ -396,7 +447,7 @@ set pgp(gpg,pat_getDecryptSym) "NEED_PASSPHRASE_SYM"
 
 ###############
 # Exec_Decrypt
-set pgp(gpg,args_decrypt) {-o $out $in}
+set pgp(gpg,args_decrypt) {--status-fd 2 -o $out $in}
 
 ##################### >>>>>>>>>>>> DELETE
 # Exec_DecryptExpect
@@ -460,6 +511,4 @@ set pgp(gpg,cmd_User) {
 ##################
 # WWW_QueryHKPKey
 set pgp(gpg,args_HKPimport) {--keyserver $server --recv-keys 0x$id}
-
-###
 }
