@@ -91,7 +91,7 @@ you want to use to generate your key?"]
 
     foreach v $wantToInstall {
 	# setup directory
-	catch { exec mkdir [set pgp($v,defaultPath)] }
+	catch { file mkdir [set pgp($v,defaultPath)] }
 	# setup config file
 	catch { exec touch [set pgp($v,configFile)] }
 	# make the key pair(s) and self sign it/them
@@ -504,7 +504,10 @@ proc Pgp_Process { v srcfile dstfile } {
     set msg [open $msgfile w 0600]
     foreach line $pgpheaders { puts $msg [Pgp_Misc_FixHeader $line] }
     if {[llength $pgpheaders] > 0} { puts $msg "" }
-    puts -nonewline $msg [read $orig]
+    while {[gets $orig line] >= 0} {
+	set trimmed [string trimright $line]
+        puts $msg $trimmed
+    }
     close $orig
     close $msg
 
