@@ -142,44 +142,56 @@ proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain}
 		SeditMsg $t $in
 		return
 	    }
-	    if [regexp "\[\x80-\xff\]" [read $in]] {
+            while {![eof $in]} {
+              set buf [read $in 8192]
+              if {[regexp "\[\x80-\xff\]" $buf]} {
 		set sedit($t,8bit) 1
+                break
+              }
 	    }
 	    close $in
 	}
-	set cmd ""
 	set uuname [file tail $file]
 	if ![regexp name= $type] {
 	    append type " ; name=\"$uuname\""
 	}
-	switch -- $encoding {
-	    base64 {   lappend cmd | $mime(encode) -b }
-	    quoted-printable {   lappend cmd | $mime(encode) -q }
-	    none {set encoding {}}
-	    x-uuencode {   lappend cmd | uuencode $uuname }
-	}
-	if {[string length $cmd] == 0} {
-	    set in [open $file]
-	} else {
-	    lappend cmd < $file
-	    Exmh_Status $cmd
-	    set in [open $cmd r]
-	}
 	if [$t compare insert <= hlimit] {
 	    $t mark set insert "hlimit +1c"
 	}
 	if {$file == $quote(filename)} {
+            # Special case for inserting '@', which is a symlink to a message
+            # Initially the encoding is text/plain, bug we auto-detect inserting QP
 	    set inheaders 1
 	    set quoted 0
+            set continuation 0
+            Exmh_Debug "SeditInsertFile insert quote file"
+            set in [open $file]
 	    while {[gets $in line] > -1} {
 		if {! $inheaders || !$quote(symlink)} {
-		    $t insert insert $sedit(pref,replPrefix)$line\n
+                    if {!$quoted} {
+                        $t insert insert $sedit(pref,replPrefix)$line\n
+                    } else {
+                        if {$continuation} {
+                            $t insert insert [mime::qp_decode $line]      ;# need leading space here?
+                        } else {
+                            $t insert insert $sedit(pref,replPrefix)[mime::qp_decode $line]
+                        }
+                        set continuation [regexp =$ $line]
+                    }
 		} else {
 		    # This simple hack doesn't work for multiparts.
+
+                    # XXX - I don't know how to get to this code block, because if
+                    # quote(symlink) is ON, then the whole Quote menu is disabled.
+                    # If quote(symlink) is OFF, then we use the code above, and then
+                    # the quoted state bit is never set. XXX
+
 		    if [regexp -nocase {^content-transfer-encoding:.*quoted-printable} $line] {
+                        Exmh_Debug "SeditInsertFile detects QP"
 			set quoted 1
 			set sedit($t,8bit) 1
 			if {$sedit($t,quote) < 0} {
+                            # Promote this part to quoted-printable
 			    set sedit($t,quote) 1
 			}
 		    }
@@ -188,21 +200,6 @@ proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain}
 		    }
 		    if {[string length $line] == 0} {
 			set inheaders 0
-			if {$quoted} {
-			    set tfile [Mime_TempFile decode]
-			    if [catch {open $tfile w} out] {
-				$t insert insert "Error: $out"
-			    } else {
-				puts -nonewline $out [read $in]
-				close $out
-				close $in
-				if [catch {open "|$mime(encode) -q -u < $tfile"} in] {
-				    $t insert insert "Error: $in"
-				    focus $t
-				    return
-				}
-			    }
-			}
 		    }
 		}
 	    }
@@ -237,7 +234,54 @@ proc SeditInsertFile { draft t file {newpart 0} {encoding {}} {type text/plain}
 	    } else {
 		set mark insert
 	    }
-	    $t insert $mark [read $in]
+
+            switch -- $encoding {
+                base64 {
+                  if {[info exist mime(encode)]} {
+		      $t insert $mark [exec $mime(encode) -b < $file]
+		  } elseif {[info exist mime(recode)]} {
+		      $t insert $mark [exec $mime(recode) data..base64 < $file]
+		  } else {
+		    Base64_EncodeInit state old length
+                    set in [open $file]
+		    fconfigure $in -encoding binary -translation binary
+                    while {![eof $in]} {
+                        $t insert $mark [Base64_EncodeBlock [read $in 4096] state old length]
+                    }
+                    $t insert $mark [Base64_EncodeTail state old length]
+                  }
+                }
+                quoted-printable {
+                  if {[info exist mime(encode)]} {
+                    $t insert $mark [exec $mime(encode) -q < $file]
+		  } elseif {[info exist mime(recode)]} {
+		    $t insert $mark [exec $mime(recode) data..qp < $file]
+                  } else {
+                    set line_cnt 0  ;# Accumulate lines before calling the encoder
+                    set buffer ""
+                    set in [open $file]
+                    while {[gets $in line] >= 0} {
+                        append buffer $line\n
+                        incr line_cnt
+                        if {$line_cnt > 1000} {
+                            $t insert $mark [::mime::qp_encode $buffer]
+                            set line_cnt 0
+                            set buffer ""
+                        }
+                    }
+                    $t insert $mark [::mime::qp_encode $buffer]
+                  }
+                }
+                x-uuencode {
+                  $t insert $mark "Error: x-uuencode attachments not supported\n"
+                }
+                "" -
+                none -
+                default {
+                  set in [open $file]
+                  $t insert $mark [read $in]
+                }
+            }
 	}
 	catch {close $in}
 	catch {close $filein}
@@ -391,7 +435,7 @@ proc Sedit_FormatParagraph { t } {
 
 proc SeditInsertFileDirect { draft t } {
     global sedit
-    set name [FSBox "Select file name"]
+    set name [FSBox "Select file name" "" read]
     if {$name != ""} {
 	if [file readable $name] {
 	    # check for 8bit characters in the file
@@ -410,7 +454,7 @@ proc SeditInsertFileDirect { draft t } {
 }
 proc SeditInsertFileDialog { draft t } {
     global sedit
-    set name [FSBox "Select file name"]
+    set name [FSBox "Select file name" "" read]
     if {$name != ""} {
 	if [file readable $name] {
 	    set options [SeditFormatDialog $t $name]
@@ -459,7 +503,7 @@ proc SeditInsertExternalDialog { draft t } {
 	set name {}
 	set options [SeditExternalUrlDialog $t $name]
     } else {
-        set name [FSBox "(Optionally) Select file name"]
+        set name [FSBox "(Optionally) Select file name" "" read]
         set options [SeditExternalFileDialog $t $name]
         }
     set tmpfname [Mime_TempFile extern]
@@ -634,7 +678,7 @@ proc SeditFormatNewPart { t f {doit 0} } {
     destroy $f
 }
 proc SeditSpell { draft f t } {
-    global sedit editor wish
+    global sedit editor wish argv0
     set parent [file root $f]
     if {[winfo exists $parent.spell]} {
 	destroy $parent.spell
@@ -644,12 +688,16 @@ proc SeditSpell { draft f t } {
     set path [Env_Tmp]/exmh.s[pid].[file tail $t]
     SeditSaveBody $t $path
 
+    set async_hack 0
     switch -- $sedit(spell) {
-	ispell {set prog {exmh-async xterm -e ispell}}
+	ispell {set prog "exmh-async xterm -e ispell" ; set async_hack 1}
 	custom {set prog $editor(spell)}
 	default {set prog spell}
     }
-    if [string match exmh-async* $prog] {
+    if {[regexp ^exmh-async $prog]} {
+	set async_hack 1
+    }
+    if {$async_hack} {
 	# exmh-async isn't really right
 	# craft a wish script instead
 	set script [Env_Tmp]/exmh.w[pid].[file tail $t]
@@ -659,11 +707,11 @@ proc SeditSpell { draft f t } {
 	}
 	puts $out "wm withdraw ."
 	puts $out "catch \{"
-	puts $out "exec [lrange $editor(spell) 1 end] $path"
+	puts $out "exec [lrange $prog 1 end] $path"
 	puts $out "\}"
 	puts $out [list send [winfo name .] [list SeditReplaceBody $t $path]]
-	puts $out "exec rm -f $path"
-	puts $out "exec rm -f $script"
+	puts $out "file delete -force $path"
+	puts $out "file delete -force $script"
 	puts $out exit
 	close $out
 	exec $wish -f $script &
@@ -671,7 +719,7 @@ proc SeditSpell { draft f t } {
     }
     # Display the results of the spell program
     catch {eval exec $prog {$path}} result
-    catch {exec rm $path}
+    catch {file delete -force $path}
 
     set f2 [Widget_Frame $parent spell {top fill}]
 
@@ -1034,7 +1082,7 @@ proc SeditAttachQuotedMessage { draft t name } {
 
 proc SeditInsertMessageDialog { draft t } {
     global sedit msg
-    set name [FSBox "Select message" $msg(path)]
+    set name [FSBox "Select message" $msg(path) read]
     if {$name != ""} {
 	if [file readable $name] {
 	    set options [SeditFormatDialog $t $name]
